/*
 * RxC1.c:
 * Control program for Lander 2
 * compile with: gcc RxC1.c -I/usr/local/include -L/usr/local/lib -lwiringPi -lpigpio -lm -o RxC1
 * Details on: https://homofaciens.de/technics-robots-Lander2-construction_en.htm
 */



//pigpio numbering varries depending on hardware version of Raspberry

#include <wiringPi.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>
#include <string.h>

//#include <sys/ioctl.h>
//#include <fcntl.h>
//#include <sys/stat.h>
//#include <sys/types.h>
#include <unistd.h>
//#include <wiringPiI2C.h>// Activate I2C on Raspberry or else compillation will fail with segmentation fault !!!

#include <pigpio.h>
//#include <time.h>
#include <sys/time.h>


//Pin numbers WiringPi
#define LIGHT_1           26//P1-32
#define LIGHT_2           29//P1-36
#define LIGHT_3           28//P1-38
#define LIGHT_4           27//P1-40



//Pin numbers pigpio !!!
#define SERVO_APERTURE_W  23//P1-16
#define SERVO_APERTURE_T  24//P1-18
#define SERVO_FOCUS_T     25//P1-22
#define SERVO_TILT         7//P1-26
#define SERVO_ROTATE      26//P1-37


#define SERVO_TILT_MIN          49
#define SERVO_TILT_MAX         101

#define SERVO_ROTATE_MIN         9
#define SERVO_ROTATE_MAX       171

#define SERVO_APERTURE_W_MIN     9
#define SERVO_APERTURE_W_MAX   171

#define SERVO_APERTURE_T_MIN     9
#define SERVO_APERTURE_T_MAX   171

#define SERVO_FOCUS_T_MIN        9
#define SERVO_FOCUS_T_MAX      171

#define SERVO_PAUSE          10000

long long getmillis(){
  struct timeval tv;
  gettimeofday(&tv, NULL);
  return (tv.tv_sec) * 1000 + (tv.tv_usec)/1000;
}

static void usage(){
  printf("usage:\n");
  printf("Turn on headlights: RxC1 h (0-3)\n");
  printf("Driving backwards: RxC1 b (0-10000)\n");
}

long checkCommands(char *commandFileName){
  long result = -1;
  char commandsLine[1000];
  FILE *commandsFile;
  int i;
  char a;
  char *pEnd;
  char systemCall[1000];
  strcpy(systemCall, "cp ");
  strcat(systemCall, commandFileName);
  strcat(systemCall, " command.tmp");

  if(access(commandFileName, W_OK) != -1){
    system(systemCall);
      if(access("command.tmp", W_OK) != -1){
        //sleep(1);
        if (!(commandsFile = fopen("command.tmp", "r"))){
          printf("Can't open command.tmp!\n");
          return -1;
        }
        else{
          a = 0;
          i = 0;
          commandsLine[0] = '\0';
          while(!(feof(commandsFile)) && i < 1000){
            fread(&a, 1, 1, commandsFile);
            commandsLine[i] = a;
            commandsLine[i+1] = '\0';
            i++;
            //printf("'%c'", a);
          }
          //printf("commandsLine='%s'\n", commandsLine);
          result = strtol(commandsLine, &pEnd, 10);
          fclose(commandsFile);
          system("rm command.tmp");
        }
      }
      else{
        printf("Can't access command.tmp!\n");
      }

  }

  //printf("%s command = %ld\n", commandFileName, result);
  return result;
}

//######################################################################
//######################################################################
//######################################################################

int main (int argc, char **argv){
  int i;

  int TiltServo = -1;
  int RotateServo = -1;
  int ApertureServoW = -1;
  int ApertureServoT = -1;
  int FocusServoT = -1;
  long command = 0;

  system("rm RxC1-command-*.txt");



  if (wiringPiSetup () == -1){
    printf("Could not set up wiringPi!");
    exit(1);
  }

  if (gpioInitialise() < 0){
    printf("Could not run gpioInitialise!");
    exit(1);
  }

  pinMode (LIGHT_1, OUTPUT);
  pinMode (LIGHT_2, OUTPUT);
  pinMode (LIGHT_3, OUTPUT);
  pinMode (LIGHT_4, OUTPUT);


  while(1){


    if((command = checkCommands("RxC1-command-servo-tilt.txt")) > -1){
      if(command > SERVO_TILT_MIN && command < SERVO_TILT_MAX){
        command *= 10;
        if(TiltServo == -1){
          TiltServo = command;
          gpioServo(SERVO_TILT, (TiltServo) * 100 / 90 + 500);
        }
        while(TiltServo != command){
          if(TiltServo > command){
            TiltServo--;
          }
          if(TiltServo < command){
            TiltServo++;
          }
          gpioServo(SERVO_TILT, (TiltServo) * 100 / 90 + 500);
          usleep(SERVO_PAUSE);
        }
      }
      else if(command == 0){
        gpioServo(SERVO_TILT, 0);
      }
      else{
        printf("TiltServo=%ld out of range!\n", command);
      }
    }

    if((command = checkCommands("RxC1-command-servo-rotate.txt")) > -1){
      if(command > SERVO_ROTATE_MIN && command < SERVO_ROTATE_MAX){
        command *= 10;
        if(RotateServo == -1){
          RotateServo = command;
          gpioServo(SERVO_ROTATE, (RotateServo) * 100 / 90 + 500);
        }
        while(RotateServo != command){
          if(RotateServo > command){
            RotateServo--;
          }
          if(RotateServo < command){
            RotateServo++;
          }
          gpioServo(SERVO_ROTATE, (RotateServo) * 100 / 90 + 500);
          usleep(SERVO_PAUSE);
        }
      }
      else if(command == 0){
        gpioServo(SERVO_ROTATE, 0);
      }
      else{
        printf("RotateServo=%ld out of range!\n", command);
      }
    }


    if((command = checkCommands("RxC1-command-servo-focus-tele.txt")) > -1){
      if(command > SERVO_FOCUS_T_MIN && command < SERVO_FOCUS_T_MAX){
        gpioServo(SERVO_FOCUS_T, (command) * 1000 / 90 + 500);
      }
      else if(command == 0){
        gpioServo(SERVO_FOCUS_T, 0);
      }
      else{
        printf("Focus Servo Tele = %ld out of range!\n", command);
      }
    }

    if((command = checkCommands("RxC1-command-servo-aperture-tele.txt")) > -1){
      if(command > SERVO_APERTURE_T_MIN && command < SERVO_APERTURE_T_MAX){
        gpioServo(SERVO_APERTURE_T, (180-command) * 1000 / 90 + 500);
      }
      else if(command == 0){
        gpioServo(SERVO_APERTURE_T, 0);
      }
      else{
        printf("Aperture Servo Tele = %ld out of range!\n", command);
      }
    }

    if((command = checkCommands("RxC1-command-servo-aperture-wide.txt")) > -1){
      if(command > SERVO_APERTURE_W_MIN && command < SERVO_APERTURE_W_MAX){
        gpioServo(SERVO_APERTURE_W, (180-command) * 1000 / 90 + 500);
      }
      else if(command == 0){
        gpioServo(SERVO_APERTURE_W, 0);
      }
      else{
        printf("Aperture Servo Wide Angle = %ld out of range!\n", command);
      }
    }

    if((command = checkCommands("RxC1-command-headlight.txt")) > -1){
      if(command == 0){
        digitalWrite(LIGHT_1, 0);
        digitalWrite(LIGHT_2, 0);
        digitalWrite(LIGHT_3, 0);
        digitalWrite(LIGHT_4, 0);
      }
      else if(command == 1){
        digitalWrite(LIGHT_1, 1);
        digitalWrite(LIGHT_2, 0);
        digitalWrite(LIGHT_3, 0);
        digitalWrite(LIGHT_4, 0);
      }
      else if(command == 2){
        digitalWrite(LIGHT_1, 1);
        digitalWrite(LIGHT_2, 1);
        digitalWrite(LIGHT_3, 0);
        digitalWrite(LIGHT_4, 0);
      }
      else if(command == 3){
        digitalWrite(LIGHT_1, 1);
        digitalWrite(LIGHT_2, 1);
        digitalWrite(LIGHT_3, 1);
        digitalWrite(LIGHT_4, 0);
      }
      else if(command == 4){
        digitalWrite(LIGHT_1, 1);
        digitalWrite(LIGHT_2, 1);
        digitalWrite(LIGHT_3, 1);
        digitalWrite(LIGHT_4, 1);
      }
      else{
        printf("Headlight value = %ld out of range!\n", command);
      }
    }


    usleep(500000);
  }


  return 0 ;
}













